# Utility -- a collection of some generically useful routines 

# Sorted: a sorted array.  Note that since we only redefine push, the
# 	other array insertion operations do not work correctly.
#	To be complete, I'd redefine those too, but for now, we only use push.
class Sorted < Array
	def push(anObj)
		for i in 0...self.length
			if (anObj <=> self[i]) < 0
				after = self.slice!(i...self.length)
				return self.concat([anObj]).concat(after)
			end
		end
		self.concat [anObj]		# just append at the end
	end
end

# An event is work to be invoked at a fixed time in the future.
# We sort events based on their times.
class Event
	def initialize(timeToOccur, work)
		@timeToOccur = timeToOccur
		@upcall = work		# code to invoke when event occurs
	end
	
	def <=> (anEvent)
		@timeToOccur <=> anEvent.timeToOccur
	end

	def to_s
		@timeToOccur.to_s
	end
	
	attr_reader :timeToOccur, :upcall
end

# A sorted list of events, so that they can be delivered in order.
class SortedEvents < Sorted
	def push(timeToOccur, work)
		super(Event.new(timeToOccur, work))
	end
end

# Debug -- a very simple control for printing debug messages.
class Debug
	INPUT       = 0b00001
	PHYSICAL    = 0b00010
	NETWORK     = 0b00100
	TRANSPORT   = 0b01000
	APPLICATION = 0b10000
	ALL         = 0b11111

	def initialize
		$debugFlags = ALL unless (defined? $debugFlags)
	end
	def print(flags, *varargs)
 		$stderr.print varargs if (flags & $debugFlags != 0)
	end
	def puts(flags, *varargs)
 		$stderr.puts varargs if (flags & $debugFlags != 0)
	end
end


# peek -- takes an array of IO objects, and returns the first that
# 	is ready to be read.  A simple wrapper around select, to
#	hide a few of the details.

def peek (readArray, timeout)
	rslt = Kernel.select(readArray, nil, nil, timeout)
	rslt ? rslt[0] : nil
end

# check if an I/O channel is at EOF
# but! do so without stalling (hence, peek first)
def iodone? (channel)
	peek([channel], 0) and channel.eof
end

# fishTime -- convert kernel reported time into an integer, in microseconds
def fishTime
	time = Time.now
	time.tv_sec*1000000 + time.usec
end

# vim: set ts=2 sw=2 noet :
